#include <OctoWS2811.h>
#include <math.h>
#include "Images.h"
#include "Color.h"

const int ledsPerStrip = 22;
const int ledPin = 13;  // blink to show we're alive

DMAMEM int displayMemory[ledsPerStrip*6];
int drawingMemory[ledsPerStrip*6];
const int config = WS2811_GRB | WS2811_800kHz;
OctoWS2811 leds(ledsPerStrip, displayMemory, drawingMemory, config);

// ledOrder[] is a tiny bit more complicated than the HypnoLamp! 127 RGB LED units,
// vaguely arranged like a 13x13 grid with some missing cells.
// Oriented with the "top" CAT6 cable running to the top of the LED hemisphere:
RGB grid[PX_WIDTH*PX_HEIGHT];

const uint8_t S0 = 0 * ledsPerStrip;
const uint8_t S1 = 1 * ledsPerStrip;
const uint8_t S2 = 2 * ledsPerStrip;
const uint8_t S3 = 3 * ledsPerStrip;
const uint8_t S4 = 4 * ledsPerStrip;
const uint8_t S5 = 5 * ledsPerStrip;
const uint8_t S6 = 6 * ledsPerStrip;
const uint8_t S7 = 7 * ledsPerStrip;
const uint8_t __nil = 0xff;  // no LED exists for this grid coordinate

PROGMEM prog_uchar  ledOrder[]  =  {
  S3+12,  __nil,  __nil,  __nil,  S4+ 0,  __nil,  __nil,  __nil,  S1+12,  __nil,  __nil,  __nil,  S7+ 0,
  S3+11,  __nil,  S2+10,  __nil,  S4+ 1,  __nil,  S5+ 0,  __nil,  S1+11,  __nil,  S5+21,  __nil,  S7+ 1,
  S3+10,  __nil,  S2+ 9,  __nil,  S4+ 2,  __nil,  S5+ 1,  __nil,  S1+10,  __nil,  S5+20,  __nil,  S7+ 2,
  S3+ 9,  S0+20,  S2+ 8,  S0+ 7,  S4+ 3,  S0+ 6,  S5+ 2,  S6+ 0,  S1+ 9,  S6+13,  S5+19,  S6+14,  S7+ 3,
  S3+ 8,  S0+19,  S2+ 7,  S0+ 8,  S4+ 4,  S0+ 5,  S5+ 3,  S6+ 1,  S1+ 8,  S6+12,  S5+18,  S6+15,  S7+ 4,
  S3+ 7,  S0+18,  S2+ 6,  S0+ 9,  S4+ 5,  S0+ 4,  S5+ 4,  S6+ 2,  S1+ 7,  S6+11,  S5+17,  S6+16,  S7+ 5,
  S3+ 6,  S0+17,  S2+ 5,  S0+10,  S4+ 6,  S0+ 3,  S5+ 5,  S6+ 3,  S1+ 6,  S6+10,  S5+16,  S6+17,  S7+ 6,
  S3+ 5,  S0+16,  S2+ 4,  S0+11,  S4+ 7,  S0+ 2,  S5+ 6,  S6+ 4,  S1+ 5,  S6+ 9,  S5+15,  S6+18,  S7+ 7,
  S3+ 4,  S0+15,  S2+ 3,  S0+12,  S4+ 8,  S0+ 1,  S5+ 7,  S6+ 5,  S1+ 4,  S6+ 8,  S5+14,  S6+19,  S7+ 8,
  S3+ 3,  S0+14,  S2+ 2,  S0+13,  S4+ 9,  S0+ 0,  S5+ 8,  S6+ 6,  S1+ 3,  S6+ 7,  S5+13,  S6+20,  S7+ 9,
  S3+ 2,  __nil,  S2+ 1,  __nil,  S4+10,  __nil,  S5+ 9,  __nil,  S1+ 2,  __nil,  S5+12,  __nil,  S7+10,
  S3+ 1,  __nil,  S2+ 0,  __nil,  S4+11,  __nil,  S5+10,  __nil,  S1+ 1,  __nil,  S5+11,  __nil,  S7+11,
  S3+ 0,  __nil,  __nil,  __nil,  S4+12,  __nil,  __nil,  __nil,  S1+ 0,  __nil,  __nil,  __nil,  S7+12,
};

//float rads[GRID_SIZE*GRID_SIZE];

#include "Test.h"
#include "SurgeUp.h"

void setup() {
  /*
  for( int i=0; i<GRID_SIZE; i++ ) {
    for( int j=0; j<GRID_SIZE; j++ ) {
      int8_t si = i - floorf(GRID_SIZE/2.0);
      int8_t sj = j - floorf(GRID_SIZE/2.0);
      rads[j*GRID_SIZE+i] = atan2f( sj, si );
      rads[j*GRID_SIZE+i] -= sqrtf( si*si + sj*sj ) * 0.25;
      rads[j*GRID_SIZE+i] *= (0.5 / M_PI);  // 1 pinwheel stripe
    }
  }
  */
  
  leds.begin();
  pinMode(ledPin, OUTPUT);
}

uint8_t blinker = 0xff;

// Earth!
//float phase = 0.0;
void loop() {
  
  
  // Pinwheel
  /*
  phase += 0.004;
  for( uint8_t i=0; i<GRID_SIZE; i++ ) {
    for( uint8_t j=0; j<GRID_SIZE; j++ ) {
      uint8_t ledIdx = pgm_read_byte(&ledOrder[j*GRID_SIZE+i]);
      if( ledIdx == __nil ) continue;  // If no LED exists at this grid coordinate, skip it
      
      float r0 = rads[j*GRID_SIZE+i] + phase;
      uint32_t rgb32 = hsv2rgb( r0, 1.0, 1.0 );
      float r1 = rads[j*GRID_SIZE+(GRID_SIZE-1-i)] + phase*0.7;
      uint32_t mirror32 = hsv2rgb( r1, 1.0, 1.0 );
      //uint32_t both32 = hsv2rgb( r0+r1, 1.0, 0.3 );
      
      leds.setPixel( ledIdx, rgb32 & ~mirror32 );
    }
  }
  */
  
  boolean doGamma = false;
  
  //testPerFrame( grid );
  surgeUpPerFrame( grid );
  
  /*
  // For each of the 13 columns: Lerp between two RGB values that are horizontally adjacent.
  for( uint8_t c=0; c<GRID_SIZE; c++ ) {
    float angle = phase*IMG_WIDTH + float(c)*(IMG_WIDTH*0.5*(1.0/GRID_SIZE));  // 0..IMG_WIDTH is a complete rotation around the image.
    float angle_floor = floor(angle);
    uint8_t leftX = uint8_t(angle_floor) % IMG_WIDTH;
    uint8_t rightX = (leftX+1) % IMG_WIDTH;
    
    // Prepare to interpolate between adjacent columns
    float lerp = angle - angle_floor;  // this column's progress from leftX to rightX (range: 0..1)
    uint8_t lerp1 = uint8_t( lerp * 0xff );  // weight towards the right column
    uint8_t lerp0 = 0xff - lerp1;            // weight towards the left column
    
    // For each row...
    for( uint8_t r=0; r<GRID_SIZE; r++ ) {
      uint8_t ledIdx = pgm_read_byte(&ledOrder[r*GRID_SIZE+c]);
      if( ledIdx == __nil ) continue;  // If no LED exists at this grid coordinate, skip it

      // Now that we know what X positions to sample from the image,
      // determine the indexes of the R,G,B values to sample
      uint16_t rgb0Idx = 3 * (IMG_WIDTH*r + leftX);
      uint16_t rgb1Idx = 3 * (IMG_WIDTH*r + rightX);
      
      uint8_t rr = ((pgm_read_byte(&IMG_DATA[rgb0Idx  ])*lerp0 + pgm_read_byte(&IMG_DATA[rgb1Idx  ])*lerp1) >> 8) & 0xff;
      rr = pgm_read_byte(&GAMMA[rr]);
      uint8_t gg = ((pgm_read_byte(&IMG_DATA[rgb0Idx+1])*lerp0 + pgm_read_byte(&IMG_DATA[rgb1Idx+1])*lerp1) >> 8) & 0xff;
      gg = pgm_read_byte(&GAMMA[gg]);
      uint8_t bb = ((pgm_read_byte(&IMG_DATA[rgb0Idx+2])*lerp0 + pgm_read_byte(&IMG_DATA[rgb1Idx+2])*lerp1) >> 8) & 0xff;
      bb = pgm_read_byte(&GAMMA[bb]);
      
      leds.setPixel( ledIdx, (rr<<16)|(gg<<8)|bb );
    }
  }
  
  phase += phaseInc;
  if( phase >= 1.0 ) {
    phase -= 1.0;
  } else if( phase < 0.0 ) {
    phase += 1.0;
  }
  */
  
  for( uint8_t j=0; j<PX_HEIGHT; j++ ) {
    for( uint8_t i=0; i<PX_WIDTH; i++ ) {
      uint8_t gridOffset = GRID(i,j);
      uint8_t ledIdx = pgm_read_byte(&ledOrder[gridOffset]);
      if( ledIdx == __nil ) continue;  // If no LED exists at this grid coordinate, skip it
      leds.setPixel( ledIdx, rgb2web( grid[gridOffset], doGamma ) );
    }
  }
  
  leds.show();
  
  blinker++;
  digitalWrite( ledPin, ((blinker==0)?HIGH:LOW) );
}

// Animation: Testing rows & columns
/*
int phase = 0;
boolean showRows = true;  // false: show columns
uint32_t color = 0x0000ff;
boolean blinker = false;

void loop() {
  int x, y;
  
  uint32_t useColor = color & 0x202020;
  
  for( x=0; x<GRID_SIZE; x++ ) {
    for( y=0; y<GRID_SIZE; y++ ) {
      uint8_t ledIdx = pgm_read_byte(&ledOrder[y*GRID_SIZE+x]);
      if( ledIdx == __nil ) continue;  // If no LED exists at this grid coordinate, skip it
      
      if( showRows ) {
        leds.setPixel( ledIdx, (y==phase) ? useColor : 0x0 );
      } else {
        leds.setPixel( ledIdx, (x==phase) ? useColor : 0x0 );
      }
    }
  }
  
  leds.show();
  
  delay(100);
  
  // Advance to the next row/column
  phase++;
  if( phase>=GRID_SIZE ) {
    showRows = !showRows;
    color = color << 8;
    if( color >= 0x01000000 ) color = 0x0000ff;
  }
  phase = (phase % GRID_SIZE);
  
  blinker = !blinker;
  digitalWrite( ledPin, (blinker?HIGH:LOW) );
}
*/



