#ifndef SURGE_UP_H__
#define SURGE_UP_H__

#include <math.h>

struct SurgeUpVars {
  SurgeUpVars() : row(0.0), color((RGB){0,0,0}), cyclesUntilColorChange(0), oscPhase(42) {
    for( uint8_t i=0; i<PX_WIDTH; i++ ) {
      oscSpeeds[i] = randRange(0.7,1.0);
    }  
  }
  float row;
  uint8_t cyclesUntilColorChange;
  RGB color;
  RGB destColor;
  float oscSpeeds[PX_WIDTH];
  float oscPhase;
};
SurgeUpVars _surgeUp;

const float SURGE_UP_SPEED = 0.0223;  // higher==faster
const float SURGE_UP_STEP = 0.85;  // 0..1 . lower==smoother stairstep transition.
const float SURGE_UP_COLOR_FADE_SPEED = 0.002;  // higher==faster
const float SURGE_UP_OSC_SPEED = 0.027;
const float SURGE_UP_OSC_RADIUS = 0.5;

void surgeUpPerFrame( RGB* grid ) {
  // Pre-cache expensive sin() computations
  _surgeUp.oscPhase += SURGE_UP_OSC_SPEED;
  
  for( uint8_t i=0; i<PX_WIDTH; i++ ) {
    // First compute the value for the top pixel (0)
    float osc = sin( _surgeUp.oscPhase * _surgeUp.oscSpeeds[i] ) * SURGE_UP_OSC_RADIUS;
    float bright = fmodf( _surgeUp.row + PX_HEIGHT + osc, PX_HEIGHT ) * (1.0/PX_HEIGHT);
    
    for( uint8_t j=0; j<PX_HEIGHT; j++ ) {
      float brt = bright;
  
      // Avoid stairstepping as rows change from light to dark
      if( brt > SURGE_UP_STEP ) {
        brt = 1.0 - ((brt-SURGE_UP_STEP) * (1.0/(1.0-SURGE_UP_STEP)));
      }
    
      //brt = 1.0 - brt;  // stripes are top-bright, bottom-dark gradients. Honestly, this looks less interesting to my eyes.
    
      RGB rgb = (RGB){ brt*_surgeUp.color.r, brt*_surgeUp.color.g, brt*_surgeUp.color.b };
    
      //grid[GRID(i,j)] = (j==_surgeUp.row) ? rgb : (RGB){0,0,0};
      grid[GRID(i,j)] = rgb;
      
      bright += (1.0/PX_HEIGHT);
      if( bright >= 1.0 ) bright -= 1.0;
    }
  }
  
  _surgeUp.row += SURGE_UP_SPEED;
  
  if( _surgeUp.cyclesUntilColorChange==0 ) {
    _surgeUp.cyclesUntilColorChange = 5;
    _surgeUp.destColor = (RGB){ randf(), randf(), randf() };
  }
  
  _surgeUp.color = lerpRGB( _surgeUp.color, _surgeUp.destColor, SURGE_UP_COLOR_FADE_SPEED );
  
  if( _surgeUp.row >= PX_HEIGHT ) {
    _surgeUp.row -= PX_HEIGHT;
    _surgeUp.cyclesUntilColorChange--;
  }
}

#endif

